# ifndef SPECIESTREE_H
# define SPECIESTREE_H

/* OVERVIEW:
 * A SpeciesTree is a mutable tree of Species.
 * A SpeciesTree has a root of type Species, and zero, one, or two
 * children, each of which is a SpeciesTree.
 * Typical SpeciesTrees are <root>, <root,child>, <root,left_child,right_child>.
 */

# include "Species.h"

# include "SpeciesTree.rh"

extern /*nullwhentrue@*/ bool SpeciesTree_isNull (SpeciesTree p_tree) /*@*/ ;

/*@constant null SpeciesTree SpeciesTree_NULL@*/
# define SpeciesTree_NULL ((SpeciesTree) NULL)

extern SpeciesTree /*@only@*/ SpeciesTree_new (/*@observer@*/ /*@dependent@*/ Species p_root)
  /* EFFECTS: Allocates and returns a new SpeciesTree, with the specified 
   *	    root and no children.
   */ ;

extern void SpeciesTree_free (/*@only@*/ SpeciesTree p_tree)
  /*@modifies p_tree@*/  
  /* MODIFIES: tree
   * EFFECTS:  Destroys tree and releases the memory it occupied.
   */ ;

extern void SpeciesTree_setLeft (SpeciesTree p_tree,
                                 /*@only@*/ SpeciesTree p_left)
 /*@modifies p_tree@*/
 /* MODIFIES: tree
  * EFFECTS:  Sets the left child of the tree.  If the tree already has a 
  * left child, the existing left child is destroyed first.
  */ ;

extern void SpeciesTree_setRight (SpeciesTree p_tree,
                                  /*@only@*/ SpeciesTree p_right)
 /*@modifies p_tree@*/
 /* MODIFIES: tree
  * EFFECTS:  Sets the right child of the tree.  If the tree already has a 
  * right child, the existing right child is destroyed first.
  */ ;

extern /*@null@*/ /*@observer@*/ SpeciesTree SpeciesTree_getLeft (SpeciesTree p_tree)
   /* EFFECTS:  Returns the left child of the tree, or NULL if there is none. */ ;

extern /*@null@*/ /*@observer@*/ SpeciesTree SpeciesTree_getRight (SpeciesTree p_tree)
   /* EFFECTS:  Returns the right child of the tree, or NULL if there is none. */ ;

extern /*@observer@*/ Species SpeciesTree_getRoot(SpeciesTree p_tree) 
  /* EFFECTS:  Returns the root of the tree. */ ;

extern /*@only@*/ char *SpeciesTree_toString (SpeciesTree p_tree) 
  /* EFFECTS: Returns a string representation of the tree. */ ;

extern /*@unused@*/ bool SpeciesTree_isEmpty (SpeciesTree p_tree)
     /* EFFECTS:  Returns true iff the tree contains no nodes */ ;

# endif
